use strict;
package MockGetLog;

sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless \%args, $class;
    return $self;
}

sub info {
    my $self = shift;
    my ($cSub) =  @_;
    $self->{'info'} = $cSub->();
}

sub getInfo {
    my $self = shift;
    return $self->{'info'};
}

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Log qw ( LogCommunication );
use Test::More;

sub testLogger {
    my ($Structure, $aFilter, $Expected) = @_;

    my $Name     = 'normal LogTest';
    my $Category = 'test';
    my $CallerDepth = 2;
    my $Object = MockGetLog->new('expected' => $Expected);
    {
        local *DE_EPAGES::Core::API::Log::GetCommunicationLog = sub {
            return $Object;
        };
        LogCommunication($Name, $Structure, $Category, $aFilter, $CallerDepth);
    }
    return $Object;
}





sub Main {
    like(
        testLogger(
            '<tagOne>eins</tagOne><tag>wurstblabla<x></x>',
            ['wurst'],
        )->getInfo(),
        qr{<tagOne>eins</tagOne><tag>$},
    'remove single word surrounded by text');

    unlike(
        testLogger(
            'davor<upss:Password>zrdz</upss:Password>dahinter',
            ['<upss:Password>.*?<\/upss:Password>'],
          )->getInfo(),
        qr{<upss:Password>zrdz</upss:Password>},
        'remove the hole XML-Tag surrounded by text'
    );

    unlike(
        testLogger(
            '<upss:Password>zrdz</upss:Password>',
            ['<upss:Password>.*?<\/upss:Password>'],
          )->getInfo(),
        qr{<upss:Password>zrdz</upss:Password>},
        'remove the hole XML-Tag which is not surrounded by text'
    );

    like(
        testLogger(
            'davor<upss:Password>zrdz</upss:Password>dahinter',
            sub {my $LogString = shift; $$LogString =~s/(?<=<ups[s]:Pas[s]word>)[^<]*?(?=<\/ups[s]:Pas[s]word>)/---replaced---/xgis;},
          )->getInfo(),
        qr{<upss:Password>---replaced---</upss:Password>}x,
        'remove only the Text in the XML-Tag which is surrounded by text'
    );

    like(
        testLogger(
            '<upss:Password>zrdz</upss:Password>',
            sub {my $LogString = shift; $$LogString =~s/(?<=<ups[s]:Pas[s]word>)[^<]*?(?=<\/ups[s]:Pas[s]word>)/---replaced---/xgis;},
          )->getInfo(),
        qr{<upss:Password>---replaced---</upss:Password>}x,
        'remove only the Text in the XML-Tag which is not surrounded by text'
    );

    done_testing();
}

RunScript( Sub => \&Main );
