use Test::More tests => 9;

use strict;
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Mail::API::SMTP;

RunScript(
    Sub => \&Main
);

sub Main {

    my $smtp = DE_EPAGES::Mail::API::SMTP->new( 'mail', 25 );  ## no critic qw(ProhibitMagicNumbers)
    ok( $smtp, 'new connection' );

    $smtp->connection( sub {
        my ($Conn) = @_;

        is( $Conn, $smtp, 'first parameter to connection()' );
        ok( $smtp->existsConnection, 'connection opened automatically' );

        $smtp->sendDataToSMTP("\ntestmail testmail", {
            'BCC' => [],
            'CC' => [],
            'FROM' => 'Milestones <Milestones@epages.de>',
            'HEADER_ENC' => 'utf-8',
            'REPLY' => undef,
            'SUBJECT' => 'testmail testmail',
            'SkipBadRecipients' => 1,
            'TO' => [
                'm.mustermann@epages.de'
                ]
                              });
        ok( 1, 'sendDataToSMTP' );
    });
    ok( !$smtp->existsConnection, 'connection closed automatically' );

    # test private function
    my $res = DE_EPAGES::Mail::API::SMTP::EncodeHeader( 'iso-8859-1', 'Mhh', 'Reply-To' );
    ok( $res eq '=?iso-8859-1?Q?M=E4=E4hh?=', 'encode header' );

    # old API
    $smtp->openConnection;
    ok( $smtp->existsConnection, 'openConnection' );

    $smtp->sendDataToSMTP("\ntestmail testmail", {
        'BCC' => [],
        'CC' => [],
        'FROM' => 'Milestones <Milestones@epages.de>',
        'HEADER_ENC' => 'utf-8',
        'REPLY' => undef,
        'SUBJECT' => 'testmail testmail',
        'SkipBadRecipients' => 1,
        'TO' => [
          'm.mustermann@epages.de'
        ]
      });
    ok( 1, 'sendDataToSMTP' );
    $smtp->closeConnection;

    ok( !$smtp->existsConnection, 'closeConnection' );
}

1;