/*global define,window,file*/

define(['testsuite/qunit', 'jquery', 'de_epages/mediagallery/filemanager/list-view', 'de_epages/mediagallery/filemanager/folder-model', '$ready!'], function (QUnit, $, ListView, FolderModel) {
    'use strict';

    QUnit.module('de_epages/mediagallery/filemanager/list-view', {
        setup: function () {
            var data = {
                    "files": [
                        {
                            "width": 1000,
                            "extension": ".jpg",
                            "name": "epages.jpg",
                            "path": "/epages.jpg",
                            "height": 237,
                            "directory": "/MediaGallery",
                            "modified": 1200994146,
                            "size": 67089,
                            "fullpath": "./epages.jpg",
                            "selectable": 0,
                            "mimetype": "image/jpg",
                            "metadata" : {
                                "type" : "MediaGallery"
                            }
                        }
                    ],
                    "children": [{
                        "files": [],
                        "children": [],
                        "metadata": {
                            "displayname": "test",
                            "type": "MediaGallery",
                            "path": "/neu/test"
                        },
                        "data": "test",
                        "state": null
                    }],
                    "metadata": {
                        "displayname": "neu",
                        "type": "MediaGallery",
                        "path": "/neu"
                    },
                    "data": "neu",
                    "state": null
                },
                model = new FolderModel(data);

            window.list = new ListView({
                model: model,
                maxRender: 1,
                multiSelect: false
            });
            $('.filemanager').append(list.render().el);

            QUnit.strictEqual(
                $('.ep-mediagallery-listView').length,
                1,
                'Create list view and append it to filemanager'
            );

            list.on('openFolder', function (data) {
                QUnit.strictEqual(
                    data.displayname,
                    'test',
                    'Trigger event open folder'
                );
            });

        },
        teardown: function () {
            $('.ep-mediagallery-listView').remove();

            QUnit.strictEqual(
                $('.ep-mediagallery-listView').length,
                0,
                'Remove tree view from filemanager container'
            );
        }
    });

    QUnit.test('view methods', 12 + 2, function () {
        $.each([
            'initialize',
            'render',
            'setSelectable',
            'addFileView',
            'addFolderView',
            'changeOrder',
            'triggerOpenFolder',
            'renameTreeNode',
            'getSelectedElements',
            'selectableselected',
            'createNewFolder',
            'deleteItems'
        ], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(list[method]),
                true,
                method + ' is a method of list view'
            );
        });
    });

    QUnit.test('view functionality', 6 + 3, function () {
        $('.ep-mediagallery-folderContainer').trigger('click');

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer.ui-selected').length,
            1,
            'Select first folder in list'
        );

        $('.ep-mediagallery-folderContainer').trigger('dblclick');

        QUnit.strictEqual(
            list.getSelectedElements().length,
            1,
            'Get selected elements: 1 item is selected'
        );

        list.deleteItems();

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer').length,
            0,
            'Delete selected folder'
        );

        $('.ep-mediagallery-fileContainer').trigger('click');


        QUnit.strictEqual(
            $('.ep-mediagallery-fileContainer.ui-selected').length,
            1,
            'Select first file in list'
        );

        list.deleteItems()

        QUnit.strictEqual(
            $('.ep-mediagallery-fileContainer').length,
            0,
            'Delete selected file'
        );

        list.createNewFolder();

        QUnit.strictEqual(
            $('.ep-mediagallery-folderContainer').length,
            1,
            'Create new folder'
        );

    });

});