/*global define,window,toolbar*/

define(['testsuite/qunit', 'jquery', 'de_epages/mediagallery/filemanager/toolbar-view', '$ready!'], function (QUnit, $, ToolbarView) {
    'use strict';

    QUnit.module('de_epages/mediagallery/filemanager/toolbar-view', {
        setup: function () {
            var options = {
                    accept: '*/*',
                    currentUsage: 12367,
                    maxSize: 100000,
                    existingFiles: [
                        {name: 'image1.png'},
                        {name: 'image2.png'},
                        {name: 'image3.png'}
                    ]
                };

            window.toolbar = new ToolbarView(options);

            $('.filemanager').append(toolbar.render().el);

            QUnit.strictEqual(
                $('.ep-mediagallery-toolbar').length,
                1,
                'Create toolbar and append it to filemanager container'
            );
        },
        teardown: function () {
            $('.ep-mediagallery-toolbar').remove();

            QUnit.strictEqual(
                $('.ep-mediagallery-toolbar').length,
                0,
                'Remove toolbar from filemanager container'
            );
        }
    });

    QUnit.test('view methods', 9 + 2, function () {
        $.each(['initialize', 'render', 'getExistingFiles', 'removeExistingFile', 'updateCurrentDirectory', 'updateUploaderData', 'changeButtonState', 'updateUsage', 'triggerNewFolder'], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(toolbar[method]),
                true,
                method + ' is a method of toolbar view'
            );
        });
    });

    QUnit.test('view functionality', 9 + 2, function () {
        var existingFiles = [
                {name: 'image1.png'},
                {name: 'image2.png'},
                {name: 'image3.png'}
            ],
            uploadData = {
                path: 'Mediagallery/test',
                files: []
            };

        QUnit.strictEqual(
            toolbar.getExistingFiles(existingFiles)[0],
            'image1.png',
            'Get name of first file in existing files array'
        );

        QUnit.strictEqual(
            toolbar.existingFiles.length,
            3,
            'Get length of existing files array'
        );

        toolbar.removeExistingFile('image2.png');

        QUnit.strictEqual(
            toolbar.existingFiles.length,
            2,
            'Get length of existing files array after one was removed'
        );

        toolbar.updateCurrentDirectory(uploadData);

        QUnit.strictEqual(
            toolbar.uploader.presentationUiUploader('option', 'data').Directory,
            'Mediagallery/test',
            'After update current directory "Mediagallery/test" is selected'
        );

        QUnit.strictEqual(
            toolbar.uploader.presentationUiUploader('option', 'exists').length,
            0,
            'After update current directory there are no existing files'
        );

        toolbar.changeButtonState(false);

        QUnit.strictEqual(
            toolbar.newFolderButton.hasClass('disabled'),
            true,
            'Button to create new folders is disabled'
        );

        toolbar.changeButtonState(true);

        QUnit.strictEqual(
            toolbar.newFolderButton.hasClass('disabled'),
            false,
            'Button to create new folders is enabled'
        );

        QUnit.strictEqual(
            toolbar.currentUsage,
            12367,
            'Get current usage of mediagallery'
        );

        toolbar.updateUsage(23776);

        QUnit.strictEqual(
            toolbar.currentUsage,
            23776,
            'Update current usage of mediagallery'
        );
    });

});