/*jslint nomen: true, plusplus: true*/
/*global define*/

define("de_epages/mediagallery/filemanager/toolbar-view", [
    "jquery",
    "ep",
    "de_epages",
    "underscore",
    "backbone",
    "$dict!../dictionary",
    "$tmpl!./toolbar-view",

    "jquery/tmpl",
    "de_epages/mediagallery/ui/memoryusage",
    "de_epages/presentation/ui/uploader"
], function ($, ep, de_epages, _, Backbone, mediagalleryDict, template) {
    "use strict";

    var tToolbar = {
        tNewFolder: mediagalleryDict.translate("NewFolder"),
        tMaxFileSize: mediagalleryDict.translate("MaxFileSize")
    },

        view = Backbone.View.extend({
            tagName: "div",
            template: template,

            events: {
                "click .ep-mediagallery-newFolderButton": "triggerNewFolder"
            },

            initialize: function () {
                this.currentUsage = this.options.currentUsage;
                this.currentDirectory = "";
                this.existingFiles = this.getExistingFiles(this.options.existingFiles);

                // set general data for file upload
                this.uploaderData = {
                    ChangeAction: "UploadFilesToMediaGallery",
                    Directory: this.currentDirectory,
                    ErrorAction: "JSONAjaxTransportFileError",
                    ObjectID: ep.config.siteId,
                    ViewAction: "JSONViewResponse",
                    handleashtml: 1
                };

                this
                    // dis- or enable the toolbar buttons according to the opened folder
                    // you can only edit items in the mediagallery folder
                    .on("folderEditable", this.changeButtonState, this)
                    // update the memory usage when something changes
                    .on("updateUsage", this.updateUsage, this);
            },

            render: function () {
                var self = this,
                    options = $.extend(tToolbar, {
                        maxFileSize: Math.floor(ep.config.maxContentLength / 1024)
                    }),
                    tmpl = self.template(options);

                self.setElement(tmpl);

                $.extend(self, $.tmplItem(tmpl).elements);

                // initialize uploader
                // interaction with the server - the upload handling should be moved to the sync method someday
                self.uploader = de_epages(self.uploaderInput).presentationUiUploader({
                    data: this.uploaderData,
                    multiple: true,
                    exists: self.existingFiles,
                    accept: this.options.accept,
//#JSCOVERAGE_IF false
                    callback: function (data, statustext) {
                        if (statustext === "success") {
                            var folderData = {
                                type: "MediaGallery",
                                path: self.currentDirectory,
                                reload: true
                            };

                            self.updateUsage(data[0].success.currentGallerySize);
                            self.trigger("openFolder", folderData);
                        }
                    }
//#JSCOVERAGE_ENDIF
                });

                // initialize memory usage
                self.memoryUsage = de_epages(self.memoryUsageContainer).mediagalleryUiMemoryusage({
                    maxSize: self.options.maxSize,
                    currentUsage: self.currentUsage
                });

                if (self.options.currentDirectory !== "MediaGallery") {
                    self.changeButtonState(false);
                }

                return self;
            },

            // get an array of the names of the existing files in the current directory
            // this is needed for the uploader to check whether a file already exists
            getExistingFiles: function (collection) {
                var filesArray = [];

                if (_.isArray(collection)) {
                    _.each(collection, function (item) {
                        filesArray.push(item.name);
                    });
                } else {
                    collection.each(function (item) {
                        filesArray.push(item.get("name"));
                    });
                }

                return filesArray;
            },

            // remove file from existing files array
            removeExistingFile: function (name) {
                var i = 0;
                for (i; i < this.existingFiles.length; i++) {
                    if (this.existingFiles[i] === name) {
                        this.existingFiles.splice(i, 1);
                    }
                }
                this.updateUploaderData();
            },

            // update uploader data and existing files when current folder changes
            updateCurrentDirectory: function (data) {
                this.uploaderData.Directory = this.currentDirectory = data.path;
                this.existingFiles = this.getExistingFiles(data.files);
                this.updateUploaderData();
            },

            // update uploader options
            updateUploaderData: function () {
                this.uploader.presentationUiUploader("option", {
                    data: this.uploaderData,
                    exists: this.existingFiles
                });
            },

            // disable toolbar buttons if an folder isn't editable
            changeButtonState: function (editable) {
                var uploadButton = this.$(".ep-mediagallery-uploadButton");

                if (editable) {
                    this.newFolderButton.removeAttr("disabled").removeClass("disabled");
                    this.uploaderInput.removeAttr("disabled");
                    uploadButton.off("click");
                } else {
                    this.newFolderButton.attr("disabled", "disabled").addClass("disabled");
                    this.uploaderInput.attr("disabled", "disabled");
                    uploadButton.on("click", function (event) {
                        event.preventDefault();
                    });
                }
            },

            // update memory usage
            updateUsage: function (value) {
                this.currentUsage = value;
                this.memoryUsage.mediagalleryUiMemoryusage("updateUsage", this.currentUsage);
            },

            // trigger event if new folder was created
            triggerNewFolder: function () {
                this.trigger("createNewFolder");
            }
        });

    return view;
});