define(['testsuite/qunit', 'jquery', 'de_epages/product/ui/appointmentpicker', 'ep/date', '$ready!'], function (QUnit, $, de_epages, ep) {

//==================================================================================//
// FAKE: Requests
//==================================================================================//

	var DayListCount = 0,
		DayListMap = [
			[1,0,0,1,0,0,2,2,2,2,0,0,0,0,1,2,2,2,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,0,2,2,2],
			[0,0,0,0,0,0,0,0,0,0,1,2,2,2,2,2,1,1,2,1,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0],
			[1,1,1,1,0,0,0,1,0,0,1,0,0,1,1,2,1,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1],
			[0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,2,2,2,2,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,2,0,1,0,0,0,0,0,0,1,1,1,1,1,1,0,2,2,2,2,2,0,0,0,0,0,0,0,0,0,0,0]
		];

	// define fake requests
	$.ajax = function( options ){
		var jqXHR	= $.Deferred(),
			data	= options.data,
			result	= {};

		// date period loop setup
		var i = new ep.Date().setFormat(data.StartDate, "S"),
			l = new ep.Date().setFormat(data.EndDate  , "S").getTime(),
			temp;

		while( i.getTime() < l ){
			temp = {
				weekday	: i.getDay(),
				add		: {},
				j		: i.clone().setHours(8),
				m		: i.clone().setHours(20).getTime()
			};

			if( temp.weekday != 0 && temp.weekday!=6  ){
				temp.add["IsAvailable"] = 1;
			}
			else{
				temp.add["IsAvailable"] = 0;
			}
			if( !data.IsAvailableOnly ){
				temp.add["DayList"] = [];

				if( (temp.DayListMap = DayListMap[ DayListCount ] || []).length ){
					temp.DayListCount = 0;

					while( temp.j.getTime() < temp.m ){
						temp.add["DayList"].push(
							[
								temp.j.getFormat("S"),
								temp.DayListMap[ temp.DayListCount ] || 0
							]
						);

						// increment DayList times
						temp.j.addMinutes(10);
						temp.DayListCount++;
					}
				}

				// increment Map of DayList settings
				DayListCount++;
			}
			// add day to result
			result[ i.getFormat("S") ] = temp.add;
			// increment Days
			i.addDate(1);
		}

		// trigger ajax success / error
		if( $.ajax.RunError ){
			jqXHR.reject(jqXHR,'error',"Error message");
			$.ajax.setRunError(false);
		}
		else{
			jqXHR.resolve(result,'success',jqXHR);
		}

		return jqXHR;
	};

	$.ajax.RunError = true;
	$.ajax.setRunError = function(error){
		$.ajax.RunError = error;
	}

//==================================================================================//
// BEGIN: TEST
//==================================================================================//

	// some globals
	var input, button, picker, tstamp,
		date = new Date(),
		getNumberOfFreeSlots = function(indexDayList, quantityAvailable, quantityRequested){
			var count = 0;

			$.each(DayListMap[indexDayList], function(i, val){
				if((quantityAvailable-val) >= quantityRequested ){
					count++;
				}
			});

			return count;
		};



	QUnit.module('de_epages/product/ui/appointmentpicker', {
		setup: function () {
			if(input === undefined){
				input = $('input[name=date]')
					.productUiAppointmentpicker({
						currentDate: {
							year:  2012,
							month: 7,
							day:   12
							// year:  date.getFullYear(),
							// month: date.getMonth()+1,
							// day:   date.getDate()
						},
						appointment: {
							id					: 0,
							quantityMin			: 1,
							quantityRequested	: 1,
							quantityAvailable	: 2,
							minutesRequested	: 20
						}
					});

				button = input
					.next()
					.trigger('click');

				picker = $('.ep-uiDatepicker-container');

				tstamp = $('.ep-uiDatepicker-daySelected a').data('tstamp');
			}

		}
	});

	QUnit.test("prototype methods", function () {
		// Check prototype methods.
		$.each( ['_create','_createTimeSelect','_createCalendars','_calendarData','_createTimeSelectData','_setTimeVal','_getTimeVal','_loadDayList','updateAppointment'], function( i, fn ){

			QUnit.equal(
				$.isFunction($.ui.productUiAppointmentpicker.prototype[fn]),
				true,
				'$.isFunction(' + fn + ')'
			);

		});
	});

	QUnit.test("check options starting with August 2012 - Requested Quantity = 1, Available Quantity = 2", function () {
		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			getNumberOfFreeSlots(0, 2 /* quantityAvailable */, 1 /* quantityRequested */),
			"Number of available slots (August 2012, Requested Quantity = 1, Available Quantity = 2)"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"Number of clickable date buttons in the first row of datepicker"
		);

		// go to next month and select an other day
		$('.ep-uiDatepicker-content a').eq(10).trigger('click');
		$('button.ep-uiDatepicker-next').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			getNumberOfFreeSlots(1, 2 /* quantityAvailable */, 1 /* quantityRequested */),
			"Number of available slots (September 2012, Requested Quantity = 1, Available Quantity = 2)"
		);
	});


	QUnit.test("check options starting with August 2012 - Requested Quantity = 2, Available Quantity = 2", function () {
		// change quantity
		input.productUiAppointmentpicker('updateAppointment', {
				quantityRequested	: 2
			});

		// go to previos month and select the first selected day again
		$('button.ep-uiDatepicker-prev').trigger('click');
		$('.ep-uiDatepicker-content a[data-tstamp="'+tstamp+'"]').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			getNumberOfFreeSlots(1, 2 /* quantityAvailable */, 2 /* quantityRequested */),
			"Number of available slots (August 2012, Requested Quantity = 2, Available Quantity = 2)"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"Number of clickable date buttons in the first row of datepicker"
		);

		// select next month and an other day
		$('.ep-uiDatepicker-next button').trigger('click');
		$('.ep-uiDatepicker-content a').eq(9).trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			getNumberOfFreeSlots(2, 2 /* quantityAvailable */, 2 /* quantityRequested */),
			"Number of available slots (September 2012, Requested Quantity = 2, Available Quantity = 2)"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"Number of clickable date buttons in the first row of datepicker"
		);

		// click today button
		$('button.ep-uiDatepicker-today').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			getNumberOfFreeSlots(0, 2 /* quantityAvailable */, 2 /* quantityRequested */),
			"Number of available slots after clicking today button (August 2012, Requested Quantity = 2, Available Quantity = 2)"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"Number of clickable date buttons in the first row of datepicker"
		);
	})


	QUnit.test("check some additional stuff in appointment picker", function () {
		// select a time
		picker.find('.ep-uiDatepicker-timeBar select option').eq(26).attr('selected',true);
		picker.find('.ep-uiDatepicker-timeBar select').trigger('change');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option:selected').text(),
			'16:00',
			"check if correct time was set after chosing an option from the time select box"
		);

		// select an other day and simulate a server error
		$.ajax.setRunError(true);
		$('.ep-uiDatepicker-content a').eq(22).trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			1,
			"force a server error by chosing an incorrect date/day for August 2012"
		);

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"Number of clickable date buttons in the first row of datepicker"
		);

		var clone = picker.find('.ep-uiDatepicker-timeBar select option').clone(true);

		QUnit.equal(
			//clone.val(),
			/dt\.\.\./.test(clone.val()),
			//"Lädt...",
			true,
			"load... text was inserted "
		);

		QUnit.equal(
			$('body > div.ui-dialog').length,
			1,
			"loading dialog was appended to body"
		);

		// close error message and open picker again
		$('.ui-dialog-titlebar-close').trigger('click');
		button.trigger('click');

		// go to next next next month and simulate server error
		$('button.ep-uiDatepicker-next').trigger('click').trigger('click');

		// force error
		$.ajax.setRunError(true);
		$('button.ep-uiDatepicker-next').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-timeBar select option').length,
			1,
			"force a server error by chosing an incorrect date/day for September 2012"
		);

		// change month and back to trigger a request
		$('button.ep-uiDatepicker-prev').trigger('click');
		$('button.ep-uiDatepicker-next').trigger('click');

		QUnit.equal(
			picker.find('.ep-uiDatepicker-content tbody tr:eq(1) a').length,
			5,
			"Number of clickable date buttons in the first row of datepicker"
		);

		clone = picker.find('.ep-uiDatepicker-timeBar select option').clone(true);

		QUnit.equal(
			clone.val(),
			"Ausgebucht",
			"chose a date that is not available"
		);

		// close error message and open picker again
		$('.ui-dialog-titlebar-close').trigger('click');
		button.trigger('click');

		// click done button
		$('.ep-uiDatepicker-done button').trigger('click');
	})
});