/*globals define*/
/*jslint nomen:true*/

define([
    'jquery',
    'testsuite/qunit',
    'de_epages/shop/linkpicker/filemanagerview',
    '$ready!'
    /**
     * linkpicker test case: additionalpagesview
     * @param  {[type]} QUnit      [description]
     * @param  {[type]} $          [description]
     * @param  {[type]} Linkpicker [description]
     * @return {[type]}            [description]
     */
], function ($, QUnit, Filemanagerview) {
    'use strict';

    var container = $('#qunit-fixture'),
        data = {},
        view = {},
        filemanagerview = new Filemanagerview(),

        fileManager = [{
                "currentGallerySize": "1344",
                "children": [{
                        "files": [{
                                "width": null,
                                "extension": ".swf",
                                "name": "promo.swf",
                                "path": "promo.swf",
                                "height": null,
                                "directory": "/MediaGallery/",
                                "modified": 1200994146,
                                "size": 67089,
                                "fullpath": "/WebRoot/promo.swf",

                                "selectable": 1,
                                "mimetype": "application/x-shockwave-flash"
                }, {
                                "width": null,
                                "extension": ".png",
                                "name": "promo.png",
                                "path": "/promo.png",
                                "height": null,
                                "directory": "/MediaGallery",
                                "modified": 1200994146,
                                "size": 67089,
                                "fullpath": "/WebRoot/promo.png",
                                "selectable": 1,
                                "mimetype": "image/png"
                }],
                        "children": [],
                        "metadata": {
                            "displayname": "MediaGallery",
                            "type": "MediaGallery",
                            "path": ""
                        },
                        "data": "MediaGallery",
                        "state": "closed"
            }],
                "maxGallerySize": "102400"
        }],

        _ajax = $.ajax;

    $.ajax = function (o) {

        var result = $.Deferred(),
            jqXHR = result.promise();

        if (o.data) {
            if (o.data.ViewAction === "JSONInitMediaGallery") {
                result.resolve(fileManager);
            }
        } else {
            return _ajax(o);
        }
        return result;
    };

    QUnit.module("de_epages/shop/linkpicker/filemanagerview");

    QUnit.test("Linkpicker-additionalpages Prototype", function () {

        $.each([
            'render',
            'setPreviewImage',
            'clearLinkData',
            '_getCorrectPreview',
            '_publishLinkData'], function (i, fn) {

            QUnit.deepEqual($.isFunction(filemanagerview[fn]), true, "Check whether " + fn + " is a function: $.isFunction( addidtionalpagesview.prototype['" + fn + "'] )");
        });
    });

    QUnit.test("Rendered Template and UI", function () {

        view = filemanagerview.render().$el;
        container.empty().append(view);

        QUnit.strictEqual(container.html(), '<div>' + view.html() + '</div>', 'Template is rendered correctly');

        var filePath = '/WebRoot/image.gif';
        filemanagerview.setPreviewImage(filePath);
        QUnit.strictEqual(filemanagerview.data.name, 'image.gif', 'file name (label) set correctly (image)');
        QUnit.strictEqual(filemanagerview.data.path, filePath, 'file path set correctly (image)');
        QUnit.strictEqual(filemanagerview._getCorrectPreview(filePath), '/WebRoot/image.gif', 'Returned correct image path (image)');
        QUnit.strictEqual($('.ep-linkpicker-filemanager-preview img').attr('src'), '/WebRoot/image.gif', 'file preview set correctly (image)');
        QUnit.strictEqual(filemanagerview.previewLabel.html(), 'image.gif', 'file label set correctly (image)');

        filePath = '/WebRoot/file.bin';
        filemanagerview.setPreviewImage(filePath);
        QUnit.strictEqual(filemanagerview.data.name, 'file.bin', 'file name (label) set correctly (binary file)');
        QUnit.strictEqual(filemanagerview.data.path, filePath, 'file path set correctly (binary file)');
        QUnit.strictEqual(filemanagerview._getCorrectPreview(filePath), '/WebRoot/StoreTypes/x.xx.x/Store/lib/de_epages/mediagallery/images/ico_unknowndocumentpreview.png', 'Returned correct image path (binary file)');
        QUnit.strictEqual($('.ep-linkpicker-filemanager-preview img').attr('src'), '/WebRoot/StoreTypes/x.xx.x/Store/lib/de_epages/mediagallery/images/ico_unknowndocumentpreview.png', 'file preview set correctly (binary file)');
        QUnit.strictEqual(filemanagerview.previewLabel.html(), 'file.bin', 'file label set correctly (binary file)');

        filemanagerview.clearLinkData();
        QUnit.strictEqual($('.ep-linkpicker-filemanager-preview').html(), '', 'Preview is cleared');
        QUnit.strictEqual($('.ep-linkpicker-filemanager-previewLabel').html(), '', 'Label is cleared');

        container.find('button').trigger('click');
        QUnit.notEqual($('.fileManagerDialog .fileManager'), '', 'fileManager dialog opened');

        $($('li.ep-mediagallery-fileContainer')[1]).addClass('ui-selected');
        $('.fileManagerDialog').uiDialog('option', 'buttons')[1].trigger('click');

        QUnit.strictEqual(filemanagerview.data.name, 'promo.png', 'Returned correct name');
        QUnit.strictEqual(filemanagerview.data.path, '/WebRoot/promo.png', 'Returned correct path');
        QUnit.strictEqual(filemanagerview.data.preview, '/WebRoot/promo.png', 'Returned correct peview');
    });
});