use Test::More tests => 12;

use strict;
use warnings;

use DE_EPAGES::Core::API::File qw (
    DeleteFile
    ExistsFile
    OpenFile
);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::ShopTransfer::API::Constants qw (BCP_FIELD_DELIMITER BCP_ROW_DELIMITER);
use DE_EPAGES::ShopTransfer::API::BCPEditor;

RunScript( Debug => 1, Sub => sub {

    my $FileName = 'BCPEditor.bcp';

    # create the test file
    OpenFile( '>', $FileName, sub {
        my ($FileHandle) = @_;
        while( <DATA> ) {
            chomp;
            print $FileHandle ( (map { $_.BCP_FIELD_DELIMITER } split /\|/, $_), BCP_ROW_DELIMITER );
        }
    });
    ok( ExistsFile($FileName), "ExistsFile($FileName)" );

    my $BCP = DE_EPAGES::ShopTransfer::API::BCPEditor->new( $FileName );

    # forEach
    my $Count = 0;
    $BCP->forEach( sub { $Count++ } );
    is $Count, 6, 'forEach';

    # countRows
    is $BCP->countRows( {} ), 6, 'countRows';

    # findRows
    my $aaFeatures = $BCP->findRows({ 1 => 1, 2 => 'Features'});
    is scalar @$aaFeatures, 1, 'findRows 1';
    is $aaFeatures->[0]->[2], 'Features', 'findRows 2';
    my $ParentID = $aaFeatures->[0]->[0];
    is $BCP->countRows({ 1 => $ParentID }), 2, 'findRows 3';

    # updateRows
    is scalar(@{$BCP->findRows({ 1 => 5, 2 => 'Demo' })}), 1, 'before updateRows';
    $BCP->updateRows({ 1 => 5, 2 => 'Demo'}, sub {
        my @Values = @_;
        $Values[2] = 'Ecom1000';
        return @Values;
    });
    is $BCP->countRows({ 1 => 5, 2 => 'Demo' }), 0, 'after updateRows 1';
    is $BCP->countRows({ 1 => 5, 2 => 'Ecom1000' }), 1, 'after updateRows 2';

    # deleteRows
    is $BCP->countRows({ 1 => 2 }), 2, 'before deleteRows';
    $BCP->deleteRows({ 1 => 2});
    is $BCP->countRows({ 1 => 2 }), 0, 'after deleteRows 1';
    is $BCP->countRows({}), 4, 'after deleteRows 2';

    # cleanup
    DeleteFile( $FileName );
});

__DATA__
1||System
2|1|Features
3|2|Ekomi
4|2|Ebay
5|1|ShopTypes
6|5|Demo
