use strict;
use Test::More;
use File::Find qw ( find );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw ( ExistsDirectory GetSubDirectories );
use DE_EPAGES::XML::API::TestImportHandler qw (TestFile);


# these files are ignored because the xml syntax is wrong (TLEs used etc.)
my @IgnoreErrorList = qw( DE_EPAGES/Forum/Templates/RSS/RSS-Blog.Page.xml
                          DE_EPAGES/Forum/Templates/RSS/RSS-Forum.Page.xml
                        );


sub Main {

    my $Directory = $ENV{'EPAGES_CARTRIDGES'}.'/DE_EPAGES';
    my @Files;
    my $aSubDirectories = GetSubDirectories($Directory);

    foreach my $Directory (@$aSubDirectories){

        foreach my $DirType (qw (Database Patches Templates)) {
            if (ExistsDirectory($Directory . "/$DirType")){
                find(sub {
                        return unless -f $File::Find::name;
                        return unless $File::Find::name =~ /\.xml$/;
                        foreach my $ex (@IgnoreErrorList) {
                            return if $File::Find::name =~ /$ex/;
                        }
                        push @Files, $File::Find::name;
                    },
                    $Directory . "/$DirType"
                );
            }
        }

    }

    plan tests => scalar @Files;
    foreach my $FileName (@Files) {
        ok (TestFile($FileName), $FileName);
    }
}


RunScript(Sub=>\&Main, Debug=>1);

1;